{-| Plugin for debug output. Prints the value and type of terms.
-}
module Plugins.Debug where

import Control.Monad.Trans
import Text.PrettyPrint

import Syntax.Internal
import Syntax.Pretty

import TypeChecker
import TypeChecker.Monad
import TypeChecker.Computation
import TypeChecker.UnfoldConstants
import TypeChecker.Values

import Utils.Error
import Utils.Monads

import Plugins

plugin :: SomePlugin
plugin = mkPlugin
    Plugin  { plName		= "debug"
            , plHelp            = debugHelp
	    , plInit		= \_ -> return []
	    , plFinal		= debugFinal
	    , plCheck		= debugCheck
	    , plInfer		= debugInfer
	    , plOptions		= []
	    , plDefaultOptions	= ()
	    }

debugFinal _ [] = return ()
debugFinal _ vs =
    do	liftIO $ putStrLn $ "=== Debug output " ++ replicate 53 '='
	liftIO $ print $ nest 2 $ vcat $ map pr $ reverse vs
    where
	pr v = sep  [ pretty v
		    , nest 2 $ text "value :" <+> pretty (unfoldAllConstants v)
		    , nest 2 $ text "type  :" <+> pretty (valueType v)
		    , nest 2 $ text "      :" <+> pretty (unfoldAllConstants $ valueType v)
		    ]

debugCheck _ vs [e] t =
    do	v <- evaluateTypedTerm =<< checkType e t
	return (v, v:vs)

debugInfer _ vs [e] =
    do	v <- evaluateTypedTerm =<< inferType e
	return (v, v:vs)

debugHelp = show $
    vcat    [ text "Provides facilities for printing the value and type of terms."
	    , nest 2 $ fsep $ pwords $
		    "debug-plugin(e) behaves as e but after typechecking the " ++
		    "value and type of e is printed."
	    , text ""
	    ]

