{-# OPTIONS -fglasgow-exts #-}

{-| Names (identifiers) and creating of fresh names.

A name is a string which is composed of a stem and optionally a
version (decimal number).  For example, the identifier "bla3" is
interpreted as stem "bla", version 3.  If the version is missing, it
is interpreted as "-1".  Equality on identifiers is just the literal
String equality; stem and version are only relevant for the creation
of fresh names.  To this end, a data structure 'UsedNames',
implemented as a finite map, keeps track of the next free identifier
version for each stem.  If a fresh name for a certain stem is needed,
the highest version is retrieved from UsedNames and UsedNames is
incremented for this stem.

-}
 
module Utils.Names
 ( UsedNames
 , noNamesUsed
 , fresh
 , HasNames(..)
 ) where

import Control.Monad.State
import Data.Char	    ( isDigit )

import Syntax.Name	    ( Name )
import Utils.Map
import Utils.Monads

type VName = (Int, Name)

-- Note: We need overlapping instances for this.
-- And it's not strictly neccessary.
{-instance Show VName where
    show (i, x) = show x-}

-- | stem drops the trailing digits in a name
stem :: Name -> Name
stem = reverse . dropWhile isDigit . reverse

-- | splitname "stem123" = ("stem", 123) 
splitName :: Name -> (Name, Int)
splitName x = (reverse revStem, 
	       if revVer == "" then -1
	       else read (reverse revVer))
    where (revVer, revStem) = span isDigit (reverse x)

createName :: Int -> Name -> Name
createName (-1) x = x
createName    n x = x ++ show n   -- not a good idea: x ++ show (n - 1)

type UsedNames = Map Name Int

noNamesUsed :: UsedNames
noNamesUsed = emptyMap

-- | registers the next unused version of a name
freshName :: Name -> UsedNames -> (UsedNames, Name)
-- wrong: need also variants of "_"
-- freshName x env
--    | x == dontCare = (env, x)
--    | otherwise	    = (env', createName n x')
freshName x env = (env', createName n x')
    where
	-- x'	= stem x
        (x', m) = splitName x
	l	= nameVersion x' env
        n       = max m l
	env'	= updateMap x' (n+1) env

nameVersion x env = lookupMap' (-1) x env

class HasNames n where
    getNames :: n -> UsedNames
    setNames :: UsedNames -> n -> n

fresh :: (HasNames n, MonadState n m) => Name -> m Name
fresh x =
    do	(used',x') <- freshName x <$> (getNames <$> get)
	modify (setNames used')
	return x'

instance HasNames UsedNames where
    getNames = id
    setNames = const

